package com.vctgo.system.api.domain;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.vctgo.common.core.annotation.Excel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 在第三方平台，通过Isbn号获取书籍信息对象 book_base_info
 *
 * @author caesar
 * @date 2024-06-14
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class BookBaseInfoDto implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     * 主键
     */
    @TableId
    @JsonSerialize(using = ToStringSerializer.class)
    private Long id;

    /**
     * isbn号
     */
    @Excel(name = "*ISBN")
    private String isbn;

    /**
     * 中文名称
     */
    @Excel(name = "中文名称")
    private String chineseName;

    /**
     * 英文名称
     */
    @Excel(name = "英文名称")
    private String englishName;

    /**
     * 封面
     */
    @Excel(name = "封面")
    private String cover;

    /**
     * 封底
     */
    @Excel(name = "封底")
    private String backCover;

    /**
     * 插页
     */
    @Excel(name = "插页")
    private String insertPage;

    /**
     * 文字作者（著）
     */
    @Excel(name = "文字作者", readConverterExp = "著=")
    private String author;

    /**
     * 图画作者
     */
    @Excel(name = "图画作者")
    private String pictureAuthor;

    /**
     * 译者（译）
     */
    @Excel(name = "译者", readConverterExp = "译=")
    private String translator;

    /**
     * 出版社
     */
    @Excel(name = "出版社")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long publisherId;

    /**
     * 出版社
     */
    @Excel(name = "出版社名称")
    @TableField(exist = false)
    private String publisherName;

    /**
     * 出版年月
     */
    @JsonFormat(pattern = "yyyy-MM")
    @Excel(name = "出版年月", width = 30, dateFormat = "yyyy-MM")
    private Date publishDate;

    /**
     * 开本
     */
    @Excel(name = "开本")
    private String formt;

    /**
     * 其他开门内容
     */
    @Excel(name = "其他开门内容")
    private String formtContent;

    /**
     * 是否套装（1是；2否；3不确定）
     */
    @Excel(name = "是否套装")
    private String series;

    /**
     * 套装系列名称
     */
    @Excel(name = "套装名称ID")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long seriesId;

    /**
     * 套装系列名称
     */
    @Excel(name = "套装名称")
    @TableField(exist = false)
    private String seriesName;

    /**
     * 包装类型（1精装；2平装；3不确定）
     */
    @Excel(name = "包装类型", readConverterExp = "1=精装；2平装；3不确定")
    private String binding;

    /**
     * 内容简介
     */
    @Excel(name = "内容简介")
    private String summary;

    /**
     * 附件
     */
    @Excel(name = "附件")
    private String attachment;

    /**
     * 是否删除
     */
    @Excel(name = "是否删除")
    private String isDeleted;

    /**
     * 创建人ID
     */
    @Excel(name = "创建人ID")
    @TableField(fill = FieldFill.INSERT)
    private Long createById;

    /**
     * 创建者
     */
    @TableField(fill = FieldFill.INSERT)
    private String createBy;

    /**
     * 创建时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;

    /**
     * 更新者
     */
    @TableField(fill = FieldFill.UPDATE)
    private String updateBy;

    /**
     * 更新时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(fill = FieldFill.UPDATE)
    private Date updateTime;

    /**
     * 最近更新人ID
     */
    @Excel(name = "最近更新人ID")
    @TableField(fill = FieldFill.UPDATE)
    private Long updateById;

    /**
     * 租户Id
     */
    @JsonSerialize(using = ToStringSerializer.class)
    private Long tenantId;

    /**
     * 请求参数
     */
    @ApiModelProperty(hidden = true)
    @TableField(exist = false)
    @JsonInclude(JsonInclude.Include.NON_EMPTY)
    private Map<String, Object> params = new HashMap<>();


    /**
     * 0启用，1禁用
     */
    @Excel(name = "0启用，1禁用")
    private String status;

    /**
     * 领域
     */
    @Excel(name = "领域")
    private String field;

    /**
     * 领域
     */
    @TableField(exist = false)
    @Excel(name = "领域名称")
    private String fieldName;

    /**
     * 标签库
     */
    @Excel(name = "标签库")
    private String label;

    /**
     * 核心经验
     */
    @Excel(name = "核心经验")
    private String coreExperience;

    /**
     * 3-6指南
     */
    @Excel(name = "3-6指南")
    private String guidelines;

    /**
     * 班龄段
     */

    @Excel(name = "班龄段")
    private String classAge;

    /**
     * 班龄段
     */
    @TableField(exist = false)
    @Excel(name = "班龄段名称")
    private String classAgeName;


    /**
     * 封面
     */
    @Excel(name = "封面")
    private String coverThumbnail;
}
