package com.vctgo.system.api.domain;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.vctgo.common.core.annotation.Excel;
import com.vctgo.common.core.annotation.Excel.ColumnType;
import com.vctgo.common.core.web.domain.TenantEntity;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Size;
import java.util.Set;

/**
 * 角色表 sys_role
 *
 * @author vctgo
 */
@Data
@NoArgsConstructor
@TableName("sys_role")
public class SysRole extends TenantEntity
{
    private static final long serialVersionUID = 1L;

    /** 角色ID */
    @Excel(name = "角色序号", cellType = ColumnType.NUMERIC)
    @TableId(type = IdType.AUTO)
    private Long roleId;

    /** 角色名称 */
    @Excel(name = "角色名称")
    @NotBlank(message = "角色名称不能为空")
    @Size(min = 0, max = 30, message = "角色名称长度不能超过30个字符")
    private String roleName;

    /** 角色权限 */
    @Excel(name = "角色权限")
//    @NotBlank(message = "权限字符不能为空")
//    @Size(min = 0, max = 100, message = "权限字符长度不能超过100个字符")
    private String roleKey;

    /** 角色排序 */
    @Excel(name = "角色排序")
    @NotBlank(message = "显示顺序不能为空")
    private String roleSort;

    /** 数据范围（1：所有数据权限；2：自定义数据权限；3：本部门数据权限；4：本部门及以下数据权限；5：仅本人数据权限） */
    @Excel(name = "数据范围", readConverterExp = "1=所有数据权限,2=自定义数据权限,3=本部门数据权限,4=本部门及以下数据权限,5=仅本人数据权限")
    private String dataScope;

    /** 菜单树选择项是否关联显示（ 0：父子不互相关联显示 1：父子互相关联显示） */
    private boolean menuCheckStrictly;

    /** 部门树选择项是否关联显示（0：父子不互相关联显示 1：父子互相关联显示 ） */
    private boolean deptCheckStrictly;

    /** 是否管理员角色（0 不是 1是） */
    private boolean adminRole;

    /** 角色状态（0正常 1停用） */
    @Excel(name = "角色状态", readConverterExp = "0=正常,1=停用")
    private String status;

    /** 删除标志（0代表存在 1代表删除） */
    private String delFlag;

    /** 备注 */
    private String remark;

    /** 是否默认 0 非默认 1 默认 */
    private String isDefault;

    /**
     * 创建人ID
     */
    @TableField(exist = false)
    @JsonSerialize(using = ToStringSerializer.class)
    private Long userId;

    /**
     * 创建人ID
     */
    @JsonSerialize(using = ToStringSerializer.class)
    @Excel(name = "创建人ID")
    private Long createById;

    /**
     * 最近更新人ID
     */
    @JsonSerialize(using = ToStringSerializer.class)
    @Excel(name = "最近更新人ID")
    private Long updateById;

    /**
     * 租户名称
     */
    @TableField(exist = false)
    private String tenantName;

    /**
     * 用户名称
     */
    @TableField(exist = false)
    private String createUserName;

    /**
     * 角色数量
     */
    @TableField(exist = false)
    private String roleCount;

    @TableField(exist = false)
    /** 用户是否存在此角色标识 默认不存在 */
    private boolean flag = false;

    @TableField(exist = false)
    /** 菜单组 */
    private Long[] menuIds;

    @TableField(exist = false)
    /** 部门组（数据权限） */
    private Long[] deptIds;

    /** 角色菜单权限 */
    @TableField(exist = false)
    private Set<String> permissions;

    /** 租户类型*/
    @TableField(exist = false)
    private Integer tenantType;

    /** 角色菜单权限 */
    @TableField(exist = false)
    private String timeOrder;

    /** 是否为分页 0 不是 1 是 */
    @TableField(exist = false)
    private String isPage;

    public SysRole(Long roleId)
    {
        this.roleId = roleId;
    }

    public boolean isAdmin()
    {
        return isAdmin(this.roleId);
    }

    public static boolean isAdmin(Long roleId)
    {
        return roleId != null && 1L == roleId;
    }
}
