package com.vctgo.system.api.domain;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.vctgo.common.core.annotation.Excel;
import com.vctgo.common.core.annotation.Excel.ColumnType;
import com.vctgo.common.core.annotation.Excel.Type;
import com.vctgo.common.core.annotation.Excels;
import com.vctgo.common.core.xss.Xss;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.constraints.Email;
import javax.validation.constraints.Size;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 用户对象 sys_user
 *
 * @author vctgo
 */
@Data
@Builder
@AllArgsConstructor
@NoArgsConstructor
@TableName("sys_user")
public class SysUser extends SysUserTempalte
{
    private static final long serialVersionUID = 1L;

    /** 用户ID */
    @Excel(name = "用户序号", cellType = ColumnType.NUMERIC, prompt = "用户编号")
    @TableId
    @JsonSerialize(using= ToStringSerializer.class)
    private Long userId;

    /** 部门ID */
    @Excel(name = "部门编号", type = Type.IMPORT)
    private Long deptId;

    /** 用户账号 */
    @Excel(name = "登录账号")
//    @Xss(message = "用户账号不能包含脚本字符")
//    @NotBlank(message = "用户账号不能为空")
//    @Size(min = 0, max = 30, message = "用户账号长度不能超过30个字符")
    private String userAccount;

    /** 用户账号 */
    @Excel(name = "登录名称")
//    @Xss(message = "用户账号不能包含脚本字符")
//    @NotBlank(message = "用户账号不能为空")
//    @Size(min = 0, max = 30, message = "用户账号长度不能超过30个字符")
    private String userName;

    /** 用户昵称 */
    @Excel(name = "用户名称")
    @Xss(message = "用户昵称不能包含脚本字符")
    @Size(min = 0, max = 30, message = "用户昵称长度不能超过30个字符")
    private String nickName;

    /** 用户类型 */
    @Excel(name = "用户类型")
    @Xss(message = "用户类型不能包含脚本字符")
    @Size(min = 0, max = 2, message = "用户类型长度不能超过2个字符")
    private String userType;

    /** 用户在职状态 */
    @Excel(name = "用户在职状态", readConverterExp = "0=在职,1=离职")
    private String userStatus;

    /** 用户邮箱 */
    @Excel(name = "用户邮箱")
    @Email(message = "邮箱格式不正确")
    @Size(min = 0, max = 50, message = "邮箱长度不能超过50个字符")
    private String email;

    /** 手机号码 */
    @Excel(name = "手机号码")
    @Size(min = 0, max = 11, message = "手机号码长度不能超过11个字符")
    private String phonenumber;

    /** 用户性别 */
    @Excel(name = "用户性别", readConverterExp = "0=男,1=女,2=未知")
    private String sex;

    /** 用户头像 */
    private String avatar;

    /** 密码 */
    private String password;

    /** 帐号状态（0正常 1停用） */
    @Excel(name = "帐号状态", readConverterExp = "0=正常,1=停用,2=未启用")
    private String accountStatus;

    /** 帐号创建时间 */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date accountDate;

    /** 备注 */
    @Size(min = 0, max = 30, message = "备注长度不能超过30个字符")
    private String remark;

    /** 帐号状态（0正常 1停用） */
    @Excel(name = "是否为专家 0 不是 1 是", readConverterExp = "0=正常,1=停用")
    private String isExpert;

    /** 删除标志（0代表存在 1代表删除） */
    private String delFlag;

    /** 最后登录IP */
    @Excel(name = "最后登录IP", type = Type.EXPORT)
    private String loginIp;

    /** 最后登录时间 */
    @Excel(name = "最后登录时间", width = 30, dateFormat = "yyyy-MM-dd HH:mm:ss", type = Type.EXPORT)
    private Date loginDate;

    /** 部门对象 */
    @TableField(exist = false)
    @Excels({
            @Excel(name = "部门名称", targetAttr = "deptName", type = Type.EXPORT),
            @Excel(name = "部门负责人", targetAttr = "leader", type = Type.EXPORT)
    })
    private SysDept dept;

    /** 角色对象 */
    @TableField(exist = false)
    private List<SysRole> roles;

    /** 角色组 */
    @TableField(exist = false)
    private Long[] roleIds;

    /** 角色组 */
    @TableField(exist = false)
    private String role;

    /** 岗位对象 */
    @TableField(exist = false)
    private SysPost post;

    /** 岗位序号 */
    @Excel(name = "岗位序号", cellType = ColumnType.NUMERIC)
    @TableField(exist = false)
    private Long postId;

    @TableField(exist = false)
    /** 岗位组 */
    private Long[] postIds;

    @TableField(exist = false)
    /** 角色ID */
    private Long roleId;

    /**
     * 部门名称
     */
    @TableField(exist = false)
    private String deptName;

    /**
     * 岗位名称
     */
    @TableField(exist = false)
    private String postName;

    /**
     * 创建者id
     */
    @TableField(fill = FieldFill.INSERT)
    @JsonSerialize(using = ToStringSerializer.class)
    private Long createById;

    /**
     * 创建者id
     */
    @TableField(exist = false)
    private String createUserName;

    /**
     * 职级
     */
    @TableField(exist = false)
    private String professionalTitleGrade;

    @TableField(exist = false)
    private String mainPost;

    /**
     * 创建者id
     */
    @TableField(exist = false)
    private Integer tenantType;

    /**
     * 更新者id
     */
    @TableField(fill = FieldFill.INSERT_UPDATE)
    @JsonSerialize(using = ToStringSerializer.class)
    private Long updateById;

    /**
     * 租户Id列表
     */
    @TableField(exist = false)
    private List<Long> tenantIdList = new ArrayList<>();

    public SysUser(Long userId)
    {
        this.userId = userId;
    }

    public boolean isAdmin()
    {
        return isAdmin(this.userId);
    }

    public static boolean isAdmin(Long userId)
    {
        return userId != null && 1L == userId;
    }


    //===================================================================================
    @TableField(exist = false)
    private String isBackbone;

    /**
     *验证状态  0 待验证 1 通过 2 驳回
     */
    private Integer verifyStatus;

    /**
     *  数据来源1 系统 2 评选小程序
     */
    private Integer sourceType;
}
