package com.vctgo.system.api.domain.dto;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.vctgo.common.core.annotation.Excel;
import com.vctgo.common.core.web.domain.KinderInfoPlatformBaseEntity;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 幼儿园管理基本信息对象 sys_branch_kindergarten
 *
 * @author wanglonghua
 * @date 2023-06-05
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class SysBranchKindergartenDto implements Serializable {
    private static final long serialVersionUID = 1L;
    /**
     * 租户ID
     */
    private Long tenantId;

    /**
     * 主键ID
     */
    @TableId
    @JsonSerialize(using = ToStringSerializer.class)
    private Long id;

    /**
     * 幼儿园名称
     */
    private String kindergartenName;

    /**
     * 幼儿园类型
     */
    private String kindergartenType;

    /**
     * 是否集团总园
     */
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private String isGarden;

    /**
     * 集团幼儿园ID
     */
    @JsonSerialize(using = ToStringSerializer.class)
    @TableField(updateStrategy = FieldStrategy.IGNORED)
    private Long gardenId;

    /**
     * 租户账号
     */
    @TableField(exist = false)
    @JsonSerialize(using = ToStringSerializer.class)
    private Long tenantAccount;

    /**
     * 账号状态
     */
    @TableField(exist = false)
    private String accountStatus;

    /**
     * 幼儿园简称
     */
    private String simpleName;

    /**
     * 幼儿园英文名称
     */
    private String englishName;

    /**
     * 园长
     */
    private String principal;

    /**
     * 园长电话
     */
    private String principalPhone;

    /**
     * 法人
     */
    private String legalPerson;

    /**
     * 文件ID
     */
    private String fileId;

    /**
     * logo链接
     */
    private String logoUrl;

    /**
     * 所属区域编码
     */
    private String belongRegionCode;

    /**
     * 省级编码
     */
    private String provinceId;

    /**
     * 市级编码
     */
    private String cityId;

    /**
     * 区县编码
     */
    private String districtId;

    /**
     * 所属区域
     */
    private String belongRegion;

    /**
     * 详细地址
     */
    private String detailAddress;

    /**
     * 定位地址
     */
    private String locationAddress;

    /**
     * 经纬度
     */
    private String latitudeLongitude;

    /**
     * 备注
     */
    private String remark;

    /**
     * 是否删除 0 未删除 1 已删除 2 彻底删除
     */
    @TableField(fill = FieldFill.INSERT)
    @TableLogic(value = "0", delval = "1")
    private String isDeleted;

    /**
     * 创建者id
     */
    @TableField(fill = FieldFill.INSERT)
    @JsonSerialize(using = ToStringSerializer.class)
    private Long createById;

    /**
     * 更新者id
     */
    @TableField(fill = FieldFill.INSERT_UPDATE)
    @JsonSerialize(using = ToStringSerializer.class)
    private Long updateById;

    /**
     * 搜索值
     */
    @TableField(exist = false)
    @JsonIgnore
    private String searchValue;

    /**
     * 创建者
     */
    @TableField(fill = FieldFill.INSERT)
    private String createBy;

    /**
     * 创建时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;

    /**
     * 更新者
     */
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private String updateBy;

    /**
     * 更新时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

    /**
     * 请求参数
     */
    @ApiModelProperty(hidden = true)
    @TableField(exist = false)
    @JsonInclude(JsonInclude.Include.NON_EMPTY)
    private Map<String, Object> params = new HashMap<>();

}
