package com.vctgo.system.api.domain.dto;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.vctgo.common.core.annotation.Excel;
import com.vctgo.common.core.web.domain.KinderInfoPlatformBaseEntity;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 幼儿园集团管理对象 sys_garden_manage
 *
 * @author wanglonghua
 * @date 2023-06-02
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class SysGardenManageDto implements Serializable {
    private static final long serialVersionUID = 8738219754734237447L;

    /**
     * 集团id
     */
    @TableId
    @JsonSerialize(using = ToStringSerializer.class)
    private Long id;

    /**
     * 集团名称
     */
    @Excel(name = "集团名称")
    private String name;

    /**
     * 分园数量
     */
    @Excel(name = "分园数量", type = Excel.Type.EXPORT)
    private Integer branchCount;

    /**
     * 省级编码
     */
    private String provinceId;

    /**
     * 市级编码
     */
    private String cityId;

    /**
     * 所属区域
     */
    @Excel(name = "所属区域")
    private String belongRegion;

    /**
     * 区县编码
     */
    private String districtId;

    /**
     * 是否删除
     */
    private String isDeleted;

    /**
     * 创建人ID
     */
    private Long createById;

    /**
     * 最近更新人ID
     */
    private Long updateById;

    /**
     * 备注
     */
    @Excel(name = "备注", type = Excel.Type.EXPORT)
    private String remark;

    /**
     *   0 已开通  1 已禁用  2  未开通
     */
    @TableField(exist = false)
    private String accountStatus;

    /**
     * 租户ID
     */
    private Long tenantId;

    /**
     * 搜索值
     */
    @TableField(exist = false)
    @JsonIgnore
    private String searchValue;

    /**
     * 创建者
     */
    @TableField(fill = FieldFill.INSERT)
    private String createBy;

    /**
     * 创建时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;

    /**
     * 更新者
     */
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private String updateBy;

    /**
     * 更新时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

    /**
     * 请求参数
     */
    @ApiModelProperty(hidden = true)
    @TableField(exist = false)
    @JsonInclude(JsonInclude.Include.NON_EMPTY)
    private Map<String, Object> params = new HashMap<>();
}
