package com.vctgo.auth.service;

import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import com.anji.captcha.service.CaptchaCacheService;
import org.springframework.stereotype.Component;

/**
 * 自定义redis验证码缓存实现类
 *
 * @author ruoyi
 */
@Component
public class CaptchaRedisService implements CaptchaCacheService {
    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    @Autowired
    private RedisTemplate redisTemplate;

    @Override
    public void set(String key, String value, long expiresInSeconds) {
        stringRedisTemplate.opsForValue().set(key, value, expiresInSeconds, TimeUnit.SECONDS);
    }

    //设置验证码
    public void saveSmsCode(String phone, String code, long duration) {
        stringRedisTemplate.opsForValue().set(phone, code, duration, TimeUnit.MINUTES);
    }

    //获取验证码
    public String getSmsCode(String phone) {
        return stringRedisTemplate.opsForValue().get(phone);
    }

    @Override
    public boolean exists(String key) {
        return stringRedisTemplate.hasKey(key);
    }

    @Override
    public void delete(String key) {
        stringRedisTemplate.delete(key);
    }

    @Override
    public String get(String key) {
        return stringRedisTemplate.opsForValue().get(key);
    }

    @Override
    public Long increment(String key, long val) {
        return stringRedisTemplate.opsForValue().increment(key, val);
    }

    @Override
    public String type() {
        return "redis";
    }


    // 保存发送时间
    public void saveLastSendTime(String key, Long timestamp, int expireSeconds) {
        redisTemplate.opsForValue().set(key, timestamp, expireSeconds, TimeUnit.SECONDS);
    }

    // 计数加1，有效期1小时
    public void incrementSendCount(String key, int increment, int initialValue, int expireSeconds) {
        redisTemplate.opsForValue().increment(key, increment); // 更新过期时间
        redisTemplate.expire(key, expireSeconds, TimeUnit.SECONDS);
        // 如果 key 不存在，则初始化为初始值
        if (Boolean.FALSE.equals(redisTemplate.hasKey(key))) {
            redisTemplate.opsForValue().set(key, initialValue, expireSeconds, TimeUnit.SECONDS);
        } else {
            // 递增计数
            redisTemplate.opsForValue().increment(key, increment);
            // 更新过期时间
            redisTemplate.expire(key, expireSeconds, TimeUnit.SECONDS);
        }
    }

    /**
     * 获取最后一次发送短信的时间
     *
     * @param key Redis键（如sms:send:account）
     * @return 最后一次发送时间的时间戳，若不存在则返回null
     */
    public Long getLastSendTime(String key) {
        Object value = redisTemplate.opsForValue().get(key);
        if (value != null) {
            return (Long) value; // 如果存在，返回时间戳
        }
        return null; // 如果Redis中没有该key，则返回null
    }

    /**
     * 获取发送验证码的次数
     *
     * @param key Redis键（如sms:send:count:account）
     * @return 发送次数，若不存在则返回null
     */
    public Integer getSendCount(String key) {
        Object value = redisTemplate.opsForValue().get(key);
        if (value != null) {
            return (Integer) value; // 如果存在，返回计数
        }
        return null; // 如果Redis中没有该key，则返回null
    }
}