package com.vctgo.file.domain;

import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.vctgo.common.core.annotation.Excel;
import com.vctgo.common.core.web.domain.BaseEntity;
import com.vctgo.common.core.web.domain.TenantEntity;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 文件回收站基本信息对象 file_ash_info
 *
 * @author caesar
 * @date 2024-02-26
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@TableName("file_ash_info")
public class FileAshInfo implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
     * id
     */
    @TableId
    @JsonSerialize(using = ToStringSerializer.class)
    private Long id;

    /**
     * 租户ID
     */
    private Long tenantId;

    /**
     * 文件ID
     */
    @Excel(name = "文件ID")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long fileId;

    /**
     * 文件类型
     */
    @Excel(name = "文件类型")
    private String fileType;

    /**
     * 文件大小
     */
    @Excel(name = "文件大小")
    private Long fileSize;

    /**
     * 文件后缀
     */
    @Excel(name = "文件后缀")
    private String fileExtension;

    /**
     * 文件名
     */
    @Excel(name = "文件名")
    private String fileName;

    /**
     * 文件完整名称
     */
    @Excel(name = "文件完整名称")
    private String fileFullName;

    /**
     * 文件url
     */
    @Excel(name = "文件url")
    private String fileUrl;

    /**
     * 是否文件夹
     */
    @Excel(name = "是否文件夹")
    private String fileIsDirectory;

    /**
     * 备注
     */
    private String remark;

    /**
     * 是否删除
     */
    @Excel(name = "是否删除")
    private String isDeleted;

    /**
     * 创建人ID
     */
    @Excel(name = "创建人ID")
    @TableField(fill = FieldFill.INSERT)
    private Long createById;

    /**
     * 创建者
     */
    @TableField(fill = FieldFill.INSERT)
    private String createBy;

    /**
     * 创建时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm")
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;

    /**
     * 更新者
     */
    @TableField(fill = FieldFill.UPDATE)
    private String updateBy;

    /**
     * 更新时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm")
    @TableField(fill = FieldFill.UPDATE)
    private Date updateTime;

    /**
     * 最近更新人ID
     */
    @Excel(name = "最近更新人ID")
    @TableField(fill = FieldFill.UPDATE)
    private Long updateById;




    //===========================页面扩展字段===================================
    /**
     * 请求参数
     */
    @ApiModelProperty(hidden = true)
    @TableField(exist = false)
    @JsonInclude(JsonInclude.Include.NON_EMPTY)
    private Map<String, Object> params = new HashMap<>();

    /**
     * 用户Id列表
     */
    @TableField(exist = false)
    private List<Long> userIdList;

    /**
     * 文件大小值
     */
    @TableField(exist = false)
    private String fileSizeBytes;


    @TableField(exist = false)
    private String updateStartTime;

    @TableField(exist = false)
    private String updateEndTime;
}
