package com.vctgo.file.mapper;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.vctgo.common.mybatisplus.mapper.BaseMapperX;
import com.vctgo.file.domain.FileReUserOperate;
import com.vctgo.system.api.domain.FileInfo;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 文件与用户的操作关系Mapper接口
 *
 * @author caesar
 * @date 2024-01-31
 */
@Repository
public interface FileReUserOperateMapper extends BaseMapperX<FileReUserOperate> {

    /**
     * 查询文件与用户的操作关系列表-分页
     *
     * @param fileReUserOperate 文件与用户的操作关系
     * @return 文件与用户的操作关系集合
     */
    IPage<FileReUserOperate> selectFileReUserOperateList(Page page, @Param("query") FileReUserOperate fileReUserOperate);

    /**
     * 查询文件与用户的操作关系列表
     *
     * @param fileReUserOperate 文件与用户的操作关系
     * @return 文件与用户的操作关系集合
     */
    List<FileReUserOperate> selectFileReUserOperateList(@Param("query") FileReUserOperate fileReUserOperate);

    /**
     * 逻辑删除文件与用户的操作关系
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    int logicalDeleteFileReUserOperateByIds(@Param("ids") Long[] ids,
                                            @Param("query") FileReUserOperate fileReUserOperate);

    /**
     * 批量删除文件与用户的操作关系
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    int deleteFileReUserOperateByIds(Long[] ids);

    /**
     * 通过用户id和文件Id获取该用户在该文件夹的权限
     *
     * @param fileId
     * @param userId
     * @return
     */
    String selectGroupPermissionsByUserIdAndFileId(@Param("fileId") String fileId,
                                                   @Param("userId") String userId);

    /**
     * 通过用户id和文件Id列表获取该用户在该文件夹的权限
     *
     * @param fileIdList
     * @param userId
     * @return
     */
    List<FileReUserOperate> selectGroupPermissionListByUserIdAndFileId(@Param("fileIdList") List<String> fileIdList,
                                                                       @Param("userId") String userId);

    List<FileReUserOperate> selectRootFileIdByUserId(@Param("userId") Long userId);

    int deleteByFileId(@Param("fileId") String fileId);


    List<Long> selectFileIdListByUserIdAndTenantId(@Param("userId") String userId,
                                                   @Param("tenantId") String tenantId);

    List<Long> selectFileIdListFileParentIdAndTenantId(@Param("tenantId") String tenantId,
                                                       @Param("parentId") String parentId,
                                                       @Param("userId") String userId);

    List<Long> selectFileIdListByAncestors(@Param("fileIdList") List<Long> fileIdList);

    List<Long> selectFileIdByFileParentIdAndTenantId(@Param("tenantId") String tenantId,
                                                     @Param("parentId") String parentId,
                                                     @Param("userId") String userId);

    List<FileReUserOperate> selectFileByTenantIdAndFileId(@Param("tenantId") String tenantId,
                                                          @Param("fileIds") String fileIds);

    int updateFileByUserOperate(@Param("userId") String userId,
                                @Param("fileIdList") List<String> fileIdList);

    /**
     * 查询用户在租户下的所有文件
     *
     * @param userId
     * @param tenantId
     * @return
     */
    List<FileInfo> selectAncestorsByUserIdAndFileId(@Param("userId") String userId,
                                                    @Param("tenantId") String tenantId);

    List<Long> selectFileByParentIdListAndTenantId(@Param("tenantId") String tenantId,
                                                   @Param("fileIdList") List<String> fileIdList,
                                                   @Param("excludeFileIdList") List<String> excludeFileIdList);
}
