package com.vctgo.file.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.vctgo.file.domain.FileReShareInfo;
import com.vctgo.file.mapper.FileReShareInfoMapper;
import com.vctgo.file.service.IFileReShareInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.vctgo.common.core.text.Convert;
import com.vctgo.common.core.utils.ServletUtils;
import com.vctgo.common.mybatisplus.constant.MybatisPageConstants;

import java.util.List;

/**
 * 分享信息与玩具信息关联Service业务层处理
 *
 * @author caesar
 * @date 2023-08-04
 */
@Slf4j
@Service
public class FileReShareInfoServiceImpl extends ServiceImpl<FileReShareInfoMapper, FileReShareInfo> implements IFileReShareInfoService {

    @Autowired
    private FileReShareInfoMapper fileReShareInfoMapper;

    /**
     * 查询分享信息与玩具信息关联
     *
     * @param id 分享信息与玩具信息关联主键
     * @return 分享信息与玩具信息关联
     */
    @Override
    public FileReShareInfo selectFileReShareInfoById(Long id) {
        return fileReShareInfoMapper.selectById(id);
    }

    /**
     * 查询分享信息与玩具信息关联列表-分页
     *
     * @param fileReShareInfo 分享信息与玩具信息关联
     * @return 分享信息与玩具信息关联
     */
    @Override
    public IPage<FileReShareInfo> selectFileReShareInfoPage(FileReShareInfo fileReShareInfo) {
        Page mpPage = new Page(Convert.toLong(ServletUtils.getParameterToInt(MybatisPageConstants.PAGE_NUM), 1L)
                , Convert.toLong(ServletUtils.getParameterToInt(MybatisPageConstants.PAGE_SIZE), 10L));
        return fileReShareInfoMapper.selectFileReShareInfoList(mpPage, fileReShareInfo);
    }

    /**
     * 查询分享信息与玩具信息关联列表
     *
     * @param fileReShareInfo 分享信息与玩具信息关联
     * @return 分享信息与玩具信息关联
     */
    @Override
    public List<FileReShareInfo> selectFileReShareInfoList(FileReShareInfo fileReShareInfo) {
        return fileReShareInfoMapper.selectFileReShareInfoList(fileReShareInfo);
    }

    /**
     * 新增分享信息与玩具信息关联
     *
     * @param fileReShareInfo 分享信息与玩具信息关联
     * @return 结果
     */

    @Override
    public int insertFileReShareInfo(FileReShareInfo fileReShareInfo) {
        return fileReShareInfoMapper.insert(fileReShareInfo);
    }

    /**
     * 修改分享信息与玩具信息关联
     *
     * @param fileReShareInfo 分享信息与玩具信息关联
     * @return 结果
     */
    @Override
    public int updateFileReShareInfo(FileReShareInfo fileReShareInfo) {
        return fileReShareInfoMapper.updateById(fileReShareInfo);
    }

    /**
     * 批量删除分享信息与玩具信息关联
     *
     * @param ids 需要删除的分享信息与玩具信息关联主键
     * @return 结果
     */
    @Override
    public int deleteFileReShareInfoByIds(Long[] ids) {
        return fileReShareInfoMapper.deleteFileReShareInfoByIds(ids);
    }

    /**
     * 删除分享信息与玩具信息关联信息
     *
     * @param id 分享信息与玩具信息关联主键
     * @return 结果
     */
    @Override
    public int deleteFileReShareInfoById(Long id) {
        return fileReShareInfoMapper.deleteById(id);
    }

    /**
     * 批量新增分享关系
     *
     * @param fileReShareInfoList
     * @return
     */
    @Override
    public int saveBatchFileReShareInfo(List<FileReShareInfo> fileReShareInfoList) {
        return saveBatch(fileReShareInfoList) ? 1 : 0;
    }
}
