package com.vctgo.file.service.impl;

import java.util.List;

import com.vctgo.file.domain.FileShare;
import com.vctgo.file.mapper.FileShareMapper;
import com.vctgo.file.service.IFileShareService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.vctgo.common.core.text.Convert;
import com.vctgo.common.core.utils.ServletUtils;
import com.vctgo.common.mybatisplus.constant.MybatisPageConstants;

/**
 * 分享信息Service业务层处理
 *
 * @author caesar
 * @date 2023-08-04
 */
@Slf4j
@Service
public class FileShareServiceImpl implements IFileShareService {

    @Autowired
    private FileShareMapper fileShareMapper;

    /**
     * 查询分享信息
     *
     * @param id 分享信息主键
     * @return 分享信息
     */
    @Override
    public FileShare selectFileShareById(Long id) {
        return fileShareMapper.selectById(id);
    }

    /**
     * 查询分享信息列表-分页
     *
     * @param fileShare 分享信息
     * @return 分享信息
     */
    @Override
    public IPage<FileShare> selectFileSharePage(FileShare fileShare) {
        Page mpPage = new Page(Convert.toLong(ServletUtils.getParameterToInt(MybatisPageConstants.PAGE_NUM), 1L)
                , Convert.toLong(ServletUtils.getParameterToInt(MybatisPageConstants.PAGE_SIZE), 10L));
        return fileShareMapper.selectFileShareList(mpPage, fileShare);
    }

    /**
     * 查询分享信息列表
     *
     * @param fileShare 分享信息
     * @return 分享信息
     */
    @Override
    public List<FileShare> selectFileShareList(FileShare fileShare) {
        return fileShareMapper.selectFileShareList(fileShare);
    }

    /**
     * 新增分享信息
     *
     * @param fileShare 分享信息
     * @return 结果
     */

    @Override
    public int insertFileShare(FileShare fileShare) {
        return fileShareMapper.insert(fileShare);
    }

    /**
     * 修改分享信息
     *
     * @param fileShare 分享信息
     * @return 结果
     */
    @Override
    public int updateFileShare(FileShare fileShare) {
        return fileShareMapper.updateById(fileShare);
    }

    /**
     * 批量删除分享信息
     *
     * @param ids 需要删除的分享信息主键
     * @return 结果
     */
    @Override
    public int deleteFileShareByIds(Long[] ids) {
        return fileShareMapper.deleteFileShareByIds(ids);
    }

    /**
     * 删除分享信息信息
     *
     * @param id 分享信息主键
     * @return 结果
     */
    @Override
    public int deleteFileShareById(Long id) {
        return fileShareMapper.deleteById(id);
    }
}
