package com.vctgo.system.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.vctgo.common.core.exception.ServiceException;
import com.vctgo.common.core.utils.poi.ExcelUtil;
import com.vctgo.common.core.web.controller.BaseController;
import com.vctgo.common.core.web.domain.AjaxResult;
import com.vctgo.common.log.annotation.Log;
import com.vctgo.common.log.enums.BusinessType;
import com.vctgo.common.security.annotation.Logical;
import com.vctgo.common.security.annotation.RequiresPermissions;
import com.vctgo.common.security.utils.SecurityUtils;
import com.vctgo.system.domain.SysPost;
import com.vctgo.system.service.ISysPostService;
import com.vctgo.system.service.ISysTeachStaffService;
import com.vctgo.system.util.ConstantUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 岗位信息操作处理
 *
 * @author vctgo
 */
@RestController
@RequestMapping("/post")
public class SysPostController extends BaseController {
    @Autowired
    private ISysPostService postService;
    @Autowired
    private ISysTeachStaffService sysTeachStaffService;

    /**
     * 获取岗位列表
     */
    @RequiresPermissions(value = {"system:post:page", "system:staff:query"}, logical = Logical.OR)
    @GetMapping("/page")
    public AjaxResult pageList(SysPost post) {
        IPage<SysPost> pageList = postService.selectPostPage(post);
        return AjaxResult.success(pageList);
    }

    @RequiresPermissions("system:post:export")
    @PostMapping("/export")
    public void export(HttpServletResponse response, SysPost post) {
        List<SysPost> list = postService.selectPostList(post);
        ExcelUtil<SysPost> util = new ExcelUtil<SysPost>(SysPost.class);
        util.exportExcel(response, list, "岗位数据");
    }

    /**
     * 根据岗位编号获取详细信息
     */
    @RequiresPermissions("system:post:query")
    @GetMapping(value = "/{postId}")
    public AjaxResult getInfo(@PathVariable Long postId) {
        return success(postService.selectPostById(postId));
    }


    /**
     * 通过岗位ID查询岗位使用数量
     */
    @GetMapping(value = "/statics/{postId}")
    public AjaxResult staticsUserCountByPostId(@PathVariable Long postId) {
        return success(postService.countUserPostById(postId));
    }

    /**
     * 获取岗位列表
     */
    @GetMapping("/weekPlan/list")
    public AjaxResult getList(SysPost post) {
        List<SysPost> postList = postService.selectPostList(post);
        return AjaxResult.success(postList);
    }

    /**
     * 新增岗位
     */
    @RequiresPermissions("system:post:add")
    @PostMapping
    public AjaxResult add(@Validated @RequestBody SysPost post) {
        if (!postService.checkPostNameUnique(post)) {
            return error("新增岗位'" + post.getPostName() + "'失败，岗位名称已存在");
        }
//        else if (!postService.checkPostCodeUnique(post))
//        {
//            return error("新增岗位'" + post.getPostName() + "'失败，岗位编码已存在");
//        }
        post.setIsSysPost(JudgeIsSysPost(post.getPostName()));
        post.setCreateBy(SecurityUtils.getUsername());
        post.setCreateById(SecurityUtils.getUserId());
        return toAjax(postService.insertPost(post));
    }

    /**
     * 修改岗位
     */
    @RequiresPermissions("system:post:edit")
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody SysPost post) {
        if (!postService.checkPostNameUnique(post)) {
            return error("修改岗位'" + post.getPostName() + "'失败，岗位名称已存在");
        }
        //if (!postService.checkPostCodeUnique(post)) {
        //    return error("修改岗位'" + post.getPostName() + "'失败，岗位编码已存在");
        //}
        if ("1".equals(post.getStatus()) && sysTeachStaffService.countUserPostById(post.getPostId()) > 0) {
            throw new ServiceException("该岗位尚有人员绑定，不可停用");
        }
        post.setIsSysPost(JudgeIsSysPost(post.getPostName()));
        post.setUpdateBy(SecurityUtils.getUsername());
        post.setUpdateById(SecurityUtils.getUserId());
        return toAjax(postService.updatePost(post));
    }

    /**
     * 判断是否系统默认岗位
     */
    public String JudgeIsSysPost(String postName) {
        List<String> postNameList = Stream.of(ConstantUtils.IS_SYS_POST_ZBJS, ConstantUtils.IS_SYS_POST_PBJS,
                ConstantUtils.IS_SYS_POST_BYY).collect(Collectors.toList());
        String isSysPost = "N";
        if (StringUtils.isNotBlank(postName) && postNameList.contains(postName.trim())) {
            isSysPost = "Y";
        }
        return isSysPost;
    }

    /**
     * 删除岗位
     */
    @RequiresPermissions("system:post:remove")
    @DeleteMapping("/{postIds}")
    public AjaxResult remove(@PathVariable Long[] postIds) {
        return toAjax(postService.deletePostByIds(postIds));
    }

    /**
     * 获取岗位选择框列表
     */
    @GetMapping("/optionselect")
    public AjaxResult optionselect() {
        List<SysPost> posts = postService.selectPostAll();
        return success(posts);
    }
}
