package com.vctgo.system.mapper;

import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.vctgo.common.mybatisplus.mapper.BaseMapperX;
import com.vctgo.system.api.domain.SysLocationSetting;
import com.vctgo.system.api.domain.dto.SysLocationReClassDto;
import com.vctgo.system.domain.vo.SysDownLocation;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 位置设置Mapper接口
 *
 * @author caesar
 * @date 2023-05-30
 */
@Repository
public interface SysLocationSettingMapper extends BaseMapperX<SysLocationSetting> {

    /**
     * 分页查询位置设置列表--区域组合在一起
     *
     * @param sysLocationSetting 位置设置
     * @return 位置设置集合
     */
    IPage<SysLocationSetting> selectSysLocationSettingList(Page page, @Param("query") SysLocationSetting sysLocationSetting);

    /**
     * 分页查询位置设置列表--区域组合在一起
     *
     * @param sysLocationSetting 位置设置
     * @return 位置设置集合
     */
    IPage<SysLocationSetting> selectBookSysLocationSettingList(Page page, @Param("query") SysLocationSetting sysLocationSetting);

    /**
     * 分页查询位置设置列表--区域组合在一起
     *
     * @param sysLocationSetting 位置设置
     * @return 位置设置集合
     */
    IPage<SysLocationSetting> listDistinguishArea(Page page, @Param("query") SysLocationSetting sysLocationSetting);

    /**
     * 通过参数获取到所有符合条件的位置id
     *
     * @param sysLocationSetting
     * @return
     */
    List<SysLocationSetting> locationIdList(@Param("query") SysLocationSetting sysLocationSetting);

    /**
     * 查询位置设置列表
     *
     * @param sysLocationSetting 位置设置
     * @return 位置设置集合
     */
    List<SysLocationSetting> selectSysLocationSettingList(@Param("query") SysLocationSetting sysLocationSetting);

    /**
     * 查询位置设置列表
     *
     * @param sysLocationSetting 位置设置
     * @return 位置设置集合
     */
    @InterceptorIgnore(tenantLine = "1")
    List<SysLocationSetting> getSysLocationList(@Param("query") SysLocationSetting sysLocationSetting);

    /**
     * 查询位置设置列表
     *
     * @param sysLocationReClassDto 位置设置
     * @return 位置设置集合
     */
    @InterceptorIgnore(tenantLine = "1")
    List<SysLocationReClassDto> getRelatedClassList(@Param("query") SysLocationReClassDto sysLocationReClassDto);

    /**
     * 批量删除位置设置
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    int deleteSysLocationSettingByIds(Long[] ids);

    List<SysLocationSetting> locationAndAreaList(@Param("tenantId") String tenantId);

    @InterceptorIgnore(tenantLine = "1")
    List<SysLocationReClassDto> getRelatedLocationList(@Param("query") SysLocationReClassDto sysLocationReClassDto);

    @InterceptorIgnore(tenantLine = "1")
    List<SysDownLocation> getDownLocationList(@Param("query") SysLocationSetting sysLocationSetting);

    List<SysLocationSetting> getLocationSettingInfo(@Param("attribute")String attribute);
}
