package com.vctgo.system.mapper;

import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.vctgo.common.mybatisplus.mapper.BaseMapperX;
import com.vctgo.system.api.domain.SysPlatformRole;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 角色表 数据层
 *
 * @author vctgo
 */
@Repository
 public interface SysPlatformRoleMapper extends BaseMapperX<SysPlatformRole>
{
    /**
     * 根据条件分页查询角色数据
     *
     * @param role 角色信息
     * @return 角色数据集合信息
     */
    IPage<SysPlatformRole> selectRoleList(Page page,@Param("query") SysPlatformRole role);

    /**
     * 根据条件分页查询角色数据
     *
     * @param role 角色信息
     * @return 角色数据集合信息
     */
     List<SysPlatformRole> selectRoleList(@Param("query") SysPlatformRole role);

    /**
     * 根据用户ID查询角色
     *
     * @param userId 用户ID
     * @return 角色列表
     */
     @InterceptorIgnore(tenantLine = "1")
     List<SysPlatformRole> selectRolePermissionByUserId(Long userId);

    /**
     * 根据租户ID查询默认管理员角色
     *
     * @param tenantId 租户ID
     * @return 角色
     */
    @InterceptorIgnore(tenantLine = "1")
    List<SysPlatformRole> queryAdminRole(Long tenantId);

    /**
     * 查询所有角色
     *
     * @return 角色列表
     */
     List<SysPlatformRole> selectRoleAll();

    /**
     * 根据用户ID获取角色选择框列表
     *
     * @param userId 用户ID
     * @return 选中角色ID列表
     */
     List<Long> selectRoleListByUserId(Long userId);

    /**
     * 通过角色ID查询角色
     *
     * @param roleId 角色ID
     * @return 角色对象信息
     */
    SysPlatformRole selectRoleById(Long roleId);

    /**
     * 根据用户ID查询角色
     *
     * @param userName 用户名
     * @return 角色列表
     */
     List<SysPlatformRole> selectRolesByUserName(String userName);

    /**
     * 校验角色名称是否唯一
     *
     * @param roleName 角色名称
     * @return 角色信息
     */
    SysPlatformRole checkRoleNameUnique(String roleName);

    /**
     * 校验角色权限是否唯一
     *
     * @param roleKey 角色权限
     * @return 角色信息
     */
    SysPlatformRole checkRoleKeyUnique(String roleKey);

    /**
     * 修改角色信息
     *
     * @param role 角色信息
     * @return 结果
     */
     int updateRole(SysPlatformRole role);

    /**
     * 新增角色信息
     *
     * @param role 角色信息
     * @return 结果
     */
     int insertRole(SysPlatformRole role);

    /**
     * 通过角色ID删除角色
     *
     * @param roleId 角色ID
     * @return 结果
     */
     int deleteRoleById(Long roleId);

    /**
     * 批量删除角色信息
     *
     * @param roleIds 需要删除的角色ID
     * @return 结果
     */
     int deleteRoleByIds(Long[] roleIds);

    /**
     * 批量删除角色信息-根据租户
     *
     * @param ids 需要删除的租户id
     * @return 结果
     */
    @InterceptorIgnore(tenantLine = "1")
    int deleteRoleByTenantId(Long[] ids);

    /**
     * 通过租户id查询当前租户的管理员角色
     *
     * @param tenantid 角色ID
     * @return 角色对象信息
     */
    @InterceptorIgnore(tenantLine = "1")
    SysPlatformRole selectRoleByTenant(Long tenantid);

}
