package com.vctgo.system.mapper;

import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.vctgo.common.mybatisplus.mapper.BaseMapperX;
import com.vctgo.system.domain.SysPost;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 岗位信息 数据层
 *
 * @author vctgo
 */
@Repository
 public interface SysPostMapper extends BaseMapperX<SysPost>
{
    /**
     * 查询岗位数据集合-分页
     *
     * @param post 岗位信息
     * @return 岗位数据集合
     */
    @InterceptorIgnore(tenantLine = "1")
    IPage<SysPost> selectPostList(Page page,@Param("query") SysPost post);
    /**
     * 查询岗位数据集合
     *
     * @param post 岗位信息
     * @return 岗位数据集合
     */
     List<SysPost> selectPostList(@Param("query") SysPost post);

    /**
     * 查询独立幼儿园的岗位
     *
     * @return 岗位列表
     */
     @InterceptorIgnore(tenantLine = "1")
     List<SysPost> selectPostByTenantId(@Param("tenantId") Long tenantId);

    /**
     * 查询所有集团（或者有集团的幼儿园）岗位
     *
     * @return 岗位列表
     */
    @InterceptorIgnore(tenantLine = "1")
    List<SysPost> selectPostAll(@Param("tenantId") Long tenantId);

    /**
     * 查询独立幼儿园的岗位列表只使用顺序排序
     *
     * @return 岗位列表
     */
    @InterceptorIgnore(tenantLine = "1")
    List<SysPost> selectPostByPostSortAndTime(@Param("tenantId") Long tenantId);

    /**
     * 通过岗位ID查询岗位信息
     *
     * @param postId 岗位ID
     * @return 角色对象信息
     */
     @InterceptorIgnore(tenantLine = "1")
     SysPost selectPostById(Long postId);

    /**
     * 根据用户ID获取岗位选择框列表
     *
     * @param userId 用户ID
     * @return 选中岗位ID列表
     */
     @InterceptorIgnore(tenantLine = "1")
     List<Long> selectPostListByUserId(Long userId);

    /**
     * 查询用户所属岗位组
     *
     * @param userName 用户名
     * @return 结果
     */
     List<SysPost> selectPostsByUserName(String userName);

    /**
     * 删除岗位信息
     *
     * @param postId 岗位ID
     * @return 结果
     */
     @InterceptorIgnore(tenantLine = "1")
     int deletePostById(Long postId);

    /**
     * 批量删除岗位信息
     *
     * @param postIds 需要删除的岗位ID
     * @return 结果
     */
     @InterceptorIgnore(tenantLine = "1")
     int deletePostByIds(Long[] postIds);

    /**
     * 修改岗位信息
     *
     * @param post 岗位信息
     * @return 结果
     */
     @InterceptorIgnore(tenantLine = "1")
     int updatePost(SysPost post);

    /**
     * 新增岗位信息
     *
     * @param post 岗位信息
     * @return 结果
     */
     int insertPost(SysPost post);

    /**
     * 校验岗位名称
     *
     * @param postName 岗位名称
     * @return 结果
     */
     @InterceptorIgnore(tenantLine = "1")
     SysPost checkPostNameUnique(@Param("postName") String postName,
                                 @Param("tenantId") Long tenantId);

    /**
     * 校验岗位编码
     *
     * @param postCode 岗位编码
     * @return 结果
     */
     SysPost checkPostCodeUnique(String postCode);

    /**
     * 批量删除岗位信息-根据租户
     *
     * @param ids 需要删除的岗位租户id
     * @return 结果
     */
    @InterceptorIgnore(tenantLine = "1")
    int deletePostByTenantId(Long[] ids);

    /**
     * 批量删除岗位信息-根据租户
     *
     * @param ids 需要删除的岗位租户id
     * @return 结果
     */
    @InterceptorIgnore(tenantLine = "1")
    int deleteSysDefaultPost(Long[] ids);

    /**
     * 变更幼儿园的岗位到所属集团
     *
     * @param gardenId 集团Id
     * @param id 幼儿园Id
     * @return 结果
     */
    @InterceptorIgnore(tenantLine = "1")
    int multiAdjustBranchPost(@Param("gardenId") Long gardenId,
                              @Param("branchId") Long id,
                              @Param("tenantType") Long tenantType);
}
