package com.vctgo.system.mapper;

import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.vctgo.common.mybatisplus.mapper.BaseMapperX;
import com.vctgo.system.api.domain.dto.ReLocationDto;
import com.vctgo.system.api.domain.SysReLocationClass;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 位置与班级关联Mapper接口
 *
 * @author caesar
 * @date 2023-05-30
 */
@Repository
public interface SysReLocationClassMapper extends BaseMapperX<SysReLocationClass>
{

    /**
     * 查询位置与班级关联列表-分页
     *
     * @param sysReLocationClass 位置与班级关联
     * @return 位置与班级关联集合
     */
    IPage<SysReLocationClass> selectSysReLocationClassList(Page page,@Param("query") SysReLocationClass sysReLocationClass);

    /**
     * 查询位置与班级关联列表
     *
     * @param sysReLocationClass 位置与班级关联
     * @return 位置与班级关联集合
     */
    List<SysReLocationClass> selectSysReLocationClassList(@Param("query") SysReLocationClass sysReLocationClass);

    /**
     * 查询位置相关信息列表
     *
     * @param reLocationDto 位置相关信息
     * @return 位置相关信息集合
     */
    List<ReLocationDto> selectLocationInfoByParam(@Param("query") ReLocationDto reLocationDto);

    /**
     * 批量删除位置与班级关联
     *
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    int deleteSysReLocationClassByIds(Long[] ids);

    /**
     * 通过位置Id和班级Id批量删除
     *
     * @param locationId 位置id
     * @param classIds    需要删除的数据主键集合
     * @return
     */
    int updateSysReLocationClassBylocationIdAndAreaIds(@Param("locationId") Long locationId, @Param("classIds") Long[] classIds);

    /**
     * 通过班级id统计绑定的用户数
     *
     * @param classId 班级Id
     * @return 结果
     */
    @InterceptorIgnore(tenantLine = "1")
    Long staticsReLocationByClassId(@Param("classId") Long classId);

    List<SysReLocationClass> selectLocationIdByUserId(@Param("userId") String userId);

    List<SysReLocationClass> selectLocationAndClass();
}
