package com.vctgo.system.mapper;

import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.vctgo.common.mybatisplus.mapper.BaseMapperX;
import com.vctgo.system.api.domain.SysUser;
import com.vctgo.system.api.domain.dto.StaticsUserPermsDto;
import com.vctgo.system.api.domain.dto.SysUserDto;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 用户表 数据层
 *
 * @author vctgo
 */
@Repository
public interface SysUserMapper extends BaseMapperX<SysUser>
{
    /**
     * 根据条件分页查询用户列表
     *
     * @param sysUser 用户信息
     * @return 用户信息集合信息
     */
    IPage<SysUser> selectUserList(Page page,@Param("query") SysUser sysUser);
    /**
     * 根据条件查询用户列表
     *
     * @param sysUser 用户信息
     * @return 用户信息集合信息
     */
     List<SysUser> selectUserList(@Param("query") SysUser sysUser);

    /**
     * 根据条件分页查询已配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
     IPage<SysUser> selectAllocatedList(Page page,@Param("query") SysUser user);

    /**
     * 根据条件分页查询未分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    IPage<SysUser> selectUnallocatedList(Page page,@Param("query") SysUser user);

    /**
     * 通过用户名查询用户
     *
     * @param userName 用户名
     * @return 用户对象信息
     */
     @InterceptorIgnore(tenantLine = "1")
     SysUser selectUserByUserName(String userName);

    /**
     * 通过用户名查询用户
     *
     * @param tenantId 用户名
     * @return 用户对象信息
     */
     @InterceptorIgnore(tenantLine = "1")
     SysUser selectUserByTenantAccount(@Param("tenantId") Long tenantId);

    /**
     * 通过用户名查询用户
     *
     * @param userName 用户名
     * @return 用户对象信息
     */
     @InterceptorIgnore(tenantLine = "1")
     SysUser selectUserByUserAccount(@Param("tenantId") Long tenantId,
                                     @Param("userName") String userName);

    /**
     * 通过用户名查询用户
     *
     * @param params 参数集合
     * @return 用户对象信息
     */
     @InterceptorIgnore(tenantLine = "1")
     List<SysUser> selectUserListByParams(@Param("params") Map<String, Object> params);

    /**
     * 通过用户ID查询用户
     *
     * @param userId 用户ID
     * @return 用户对象信息
     */
     SysUser selectUserById(@Param("userId") Long userId);

     /**
     * 通过用户ID查询用户
     *
     * @param userId 用户ID
     * @return 用户对象信息
     */
     @InterceptorIgnore(tenantLine = "1")
     SysUser selectUserByUserId(@Param("userId") Long userId,
                                @Param("tenantId") Long tenantId);

    /**
     * 根据岗位编码查询用户列表
     *
     * @param postCode 岗位编码
     * @return 结果
     */
    IPage<SysUserDto> selectPointPostUserList(Page page,
                                              @Param("postCode") String postCode,
                                              @Param("tenantId") Long tenantId,
                                              @Param("nickName") String nickName,
                                              @Param("phoneNumber") String phoneNumber);

    /**
     * 根据租户统计各部门的用户数
     *
     * @param params 参数
     * @return 结果
     */
//    List<SysUserDto> statisticsDeptUserCount(@Param("params") Map<String, Object> params);

    /**
     * 新增用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
//     int insertUser(SysUser user);

    /**
     * 修改用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
     @InterceptorIgnore(tenantLine = "1")
     int updateUser(SysUser user);

    /**
     * 修改用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
     @InterceptorIgnore(tenantLine = "1")
     int updateTenantAccount(SysUser user);

    /**
     * 修改用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
     int updateUserAccount(SysUser user);

    /**
     * 修改集团租户或者幼儿园租户的名称
     *
     * @param user 租户信息
     * @return 结果
     */
    @InterceptorIgnore(tenantLine = "1")
    int updateGardenTenantName(SysUser user);

    /**
     * 修改集团租户或者幼儿园租户的名称
     *
     * @param user 租户信息
     * @return 结果
     */
    @InterceptorIgnore(tenantLine = "1")
    int updateBranchTenantName(SysUser user);

    /**
     * 修改用户头像
     *
     * @param userName 用户名
     * @param avatar 头像地址
     * @return 结果
     */
     int updateUserAvatar(@Param("userName") String userName, @Param("avatar") String avatar);

    /**
     * 更新部分关联表的用户昵称
     *
     * @return 结果
     */
    int updateReTableNickName(@Param("userId") Long userId,
                              @Param("nickName") String nickName);

    /**
     * 重置用户密码
     *
     * @param userName 用户名
     * @param password 密码
     * @return 结果
     */
     int resetUserPwd(@Param("userName") String userName, @Param("password") String password);

    /**
     * 通过用户ID删除用户
     *
     * @param userId 用户ID
     * @return 结果
     */
     int deleteUserById(Long userId);

    /**
     * 批量删除用户信息
     *
     * @param userIds 需要删除的用户ID
     * @return 结果
     */
     int deleteUserByIds(Long[] userIds);

    /**
     * 通过用户ID清空用户账号信息
     *
     * @param userId 用户ID
     * @return 结果
     */
    @InterceptorIgnore(tenantLine = "1")
    int clearUserAccountInfo(Long userId);

    /**
     * 校验租户名称是否唯一
     *
     * @param userName 用户名称
     * @return 结果
     */
     @InterceptorIgnore(tenantLine = "1")
     SysUser checkTenantNameUnique(@Param("userName") String userName,
                                   @Param("tenantId") Long tenantId);

    /**
     * 校验用户名称是否唯一
     *
     * @param userName 用户名称
     * @return 结果
     */
     @InterceptorIgnore(tenantLine = "1")
     SysUser checkUserNameUnique(String userName);

    /**
     * 校验手机号码是否唯一
     *
     * @param phonenumber 手机号码
     * @return 结果
     */
    @InterceptorIgnore(tenantLine = "1")
    SysUser checkPhoneUnique(String phonenumber);

    /**
     * 校验email是否唯一
     *
     * @param email 用户邮箱
     * @return 结果
     */
     SysUser checkEmailUnique(String email);

    /**
     * 判断用户是否存在指定的权限
     *
     * @param  userPermsDto 部门信息
     * @return 统计对象集合
     */
    Integer judgeUserIsExistPointPerms(@Param("query") StaticsUserPermsDto userPermsDto);


    /**
     * 批量删除用户信息-根据租户
     *
     * @param ids 需要删除的租户ID
     * @return 结果
     */
    @InterceptorIgnore(tenantLine = "1")
    int deleteUserByTenantId(Long[] ids);

    /**
     * 获取用户与角色信息
     *
     * @return
     */
    List<SysUser> selectUserAndRoleList(@Param("roleKey") String roleKey);
}
