package com.vctgo.system.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.vctgo.system.domain.SysAreaSetting;

import java.util.List;
import java.util.Map;

/**
 * 区域设置Service接口
 *
 * @author caesar
 * @date 2023-05-30
 */
public interface ISysAreaSettingService
{
    /**
     * 查询区域设置
     *
     * @param areaId 区域设置主键
     * @return 区域设置
     */
    SysAreaSetting selectSysAreaSettingByAreaId(Long areaId);

    /**
     * 查询区域设置列表-分页
     *
     * @param sysAreaSetting 区域设置
     * @return 区域设置集合
     */
    IPage<SysAreaSetting> selectSysAreaSettingPage(SysAreaSetting sysAreaSetting);

    /**
     * 查询区域设置列表
     *
     * @return 区域设置集合
     */
    List<SysAreaSetting> selectSysAreaSettingList(SysAreaSetting sysAreaSetting);

    /**
     * 查询区域设置列表
     *
     * @param sysAreaSetting 区域设置
     * @return 区域设置集合
     */
    List<SysAreaSetting> selectTenantAreaSettingList(SysAreaSetting sysAreaSetting);

    /**
     * 根据相应参数查询是否存在该区域
     *
     * @param params 参数
     * @return 区域设置集合
     */
    List<SysAreaSetting> selectAreaListByParams(Map<String, Object> params);

    /**
     * 新增区域设置
     *
     * @param sysAreaSetting 区域设置
     * @return 结果
     */
    int insertSysAreaSetting(SysAreaSetting sysAreaSetting);

    /**
     * 修改区域设置
     *
     * @param sysAreaSetting 区域设置
     * @return 结果
     */
    int updateSysAreaSetting(SysAreaSetting sysAreaSetting);

    /**
     * 逻辑删除区域
     *
     * @param ids 区域id集合
     * @return 结果
     */
    int logicalDeleteByAreaIds(Long[] ids);

    /**
     * 批量删除区域设置
     *
     * @param areaIds 需要删除的区域设置主键集合
     * @return 结果
     */
    int deleteSysAreaSettingByAreaIds(Long[] areaIds);

    /**
     * 删除区域设置信息
     *
     * @param areaId 区域设置主键
     * @return 结果
     */
    int deleteSysAreaSettingByAreaId(Long areaId);

    /**
     * 查询区域信息
     * @param locationIds
     * @return
     */
    List<SysAreaSetting> getAreaSettingInfo(String locationIds,String attribute);

    /**
     * 查询区域信息
     * @return
     */
    List<String> getAreaNameByIds(List<Long> areaIds);

    List<SysAreaSetting> selectAreaNameByIds(List<Long> areaIds);

}
