package com.vctgo.system.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.vctgo.system.api.domain.dto.SysBranchKindergartenDto;
import com.vctgo.system.domain.SysBranchKindergarten;

import java.util.List;
import java.util.Map;

/**
 * 幼儿园管理基本信息Service接口
 *
 * @author wanglonghua
 * @date 2023-06-05
 */
public interface ISysBranchKindergartenService {
    /**
     * 查询幼儿园管理基本信息
     *
     * @param id 幼儿园管理基本信息主键
     * @return 幼儿园管理基本信息
     */
    SysBranchKindergarten selectSysBranchKindergartenById(Long id);

    /**
     * 查询幼儿园管理基本信息列表-分页
     *
     * @param sysBranchKindergarten 幼儿园管理基本信息
     * @return 幼儿园管理基本信息集合
     */
    IPage<SysBranchKindergarten> selectSysBranchKindergartenPage(SysBranchKindergarten sysBranchKindergarten);

    /**
     * 查询幼儿园管理基本信息列表
     *
     * @param sysBranchKindergarten 幼儿园管理基本信息
     * @return 幼儿园管理基本信息集合
     */
    List<SysBranchKindergarten> selectSysBranchKindergartenList(SysBranchKindergartenDto sysBranchKindergarten);

    /**
     * 查询幼儿园管理基本信息列表
     *
     * @param sysBranchKindergarten 幼儿园管理基本信息
     * @return 幼儿园管理基本信息集合
     */
    List<Long> selectSysBranchKindergartenIdList(SysBranchKindergartenDto sysBranchKindergarten);

    /**
     * 查询幼儿园列表(包括独立幼儿园,有所属集团的幼儿园,集团)
     */
    List<SysBranchKindergarten> kindergartenList();

    /**
     * 根据相应参数查询是否存在该幼儿园
     *
     * @param params 参数
     * @return 幼儿园管理基本信息集合
     */
    List<SysBranchKindergarten> selectBranchListByParams(Map<String, Object> params);

    /**
     * 根据幼儿园集团ID获取幼儿园列表
     *
     * @param gardenId 幼儿园集团ID
     * @return 幼儿园管理基本信息集合
     */
    List<SysBranchKindergarten> getKindergartenListByGardenId(Long gardenId);

    /**
     * 新增幼儿园管理基本信息
     *
     * @param sysBranchKindergarten 幼儿园管理基本信息
     * @return 结果
     */
    int insertSysBranchKindergarten(SysBranchKindergarten sysBranchKindergarten);

    /**
     * 修改幼儿园管理基本信息
     *
     * @param sysBranchKindergarten 幼儿园管理基本信息
     * @return 结果
     */
    int updateSysBranchKindergarten(SysBranchKindergarten sysBranchKindergarten);

    /**
     * 修改幼儿园管理基本信息
     *
     * @param sysBranchKindergarten 幼儿园管理基本信息
     * @return 结果
     */
    int updateBranchBaseInfo(SysBranchKindergarten sysBranchKindergarten);

    /**
     * 调整幼儿园的归属关系
     *
     * @param sysBranchKindergarten 幼儿园管理基本信息
     * @return 结果
     */
    int adjustAffiliation(SysBranchKindergarten sysBranchKindergarten);

    /**
     * 批量删除幼儿园管理基本信息
     *
     * @param ids 需要删除的幼儿园管理基本信息主键集合
     * @return 结果
     */
    int deleteSysBranchKindergartenByIds(Long[] ids);

    /**
     * 逻辑删除幼儿园集团管理
     *
     * @param ids 区域id集合
     * @return 结果
     */
    int logicalDeleteByIds(Long[] ids);


    int add(SysBranchKindergartenDto sysBranchKindergartenDto);

    List<SysBranchKindergartenDto> getAll(String provinceId, String cityId, String districtId);

    SysBranchKindergartenDto getById(Long id);

    /**
     * 新增幼儿园管理基本信息
     *
     * @param sysBranchKindergarten 幼儿园管理基本信息
     * @return 结果
     */
    SysBranchKindergarten voteInsertSysBranchKindergarten(SysBranchKindergarten sysBranchKindergarten);
}
