package com.vctgo.system.service;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.vctgo.system.domain.SysPlatformPost;
import com.vctgo.system.domain.SysPost;

/**
 * 岗位信息 服务层
 *
 * @author vctgo
 */
 public interface ISysPlatformPostService
{
    /**
     * 查询岗位信息集合-分页
     *
     * @param post 岗位信息
     * @return 岗位列表
     */
    IPage<SysPlatformPost> selectPostPage(SysPlatformPost post);
    /**
     * 查询岗位信息集合
     *
     * @param post 岗位信息
     * @return 岗位列表
     */
     List<SysPlatformPost> selectPostList(SysPlatformPost post);

    /**
     * 查询所有岗位
     *
     * @return 岗位列表
     */
     List<SysPlatformPost> selectPostAll();

    /**
     * 通过岗位ID查询岗位信息
     *
     * @param postId 岗位ID
     * @return 角色对象信息
     */
    SysPlatformPost selectPostById(Long postId);

    /**
     * 根据用户ID获取岗位选择框列表
     *
     * @param userId 用户ID
     * @return 选中岗位ID列表
     */
     List<Long> selectPostListByUserId(Long userId);

    /**
     * 校验岗位名称
     *
     * @param post 岗位信息
     * @return 结果
     */
    boolean checkPostNameUnique(SysPlatformPost post);

    /**
     * 校验岗位编码
     *
     * @param post 岗位信息
     * @return 结果
     */
    boolean checkPostCodeUnique(SysPlatformPost post);

    /**
     * 通过岗位ID查询岗位使用数量
     *
     * @param postId 岗位ID
     * @return 结果
     */
     int countUserPostById(Long postId);

    /**
     * 删除岗位信息
     *
     * @param postId 岗位ID
     * @return 结果
     */
     int deletePostById(Long postId);

    /**
     * 批量删除岗位信息
     *
     * @param postIds 需要删除的岗位ID
     * @return 结果
     */
     int deletePostByIds(Long[] postIds);

    /**
     * 新增保存岗位信息
     *
     * @param post 岗位信息
     * @return 结果
     */
     int insertPost(SysPlatformPost post);

    /**
     * 修改保存岗位信息
     *
     * @param post 岗位信息
     * @return 结果
     */
     int updatePost(SysPlatformPost post);
}
