package com.vctgo.system.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.vctgo.system.api.domain.SysPlatformDept;
import com.vctgo.system.api.domain.SysPlatformUser;

import java.util.List;

/**
 * 用户 业务层
 *
 * @author vctgo
 */
 public interface ISysPlatformUserService
{
    /**
     * 根据条件分页查询用户列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    IPage<SysPlatformUser> selectUserPage(SysPlatformUser user);
    /**
     * 根据条件查询用户列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
     List<SysPlatformUser> selectUserList(SysPlatformUser user);

    /**
     * 流程管理获取用户列表
     *
     * @return
     */
    List<SysPlatformDept> processUserList();

    /**
     * 平台获取流程管理专家列表
     *
     * @return
     */
    List<SysPlatformUser> processExpertList();

    /**
     * 通过租户id获取特定的用户列表
     *
     * @param tenantIds
     * @return
     */
    List<SysPlatformUser> userListByTenantIds(String tenantIds);

    /**
     * 根据条件分页查询已分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
     IPage<SysPlatformUser> selectAllocatedList(SysPlatformUser user);

    /**
     * 根据条件分页查询未分配用户角色列表
     *
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    IPage<SysPlatformUser> selectUnallocatedList(SysPlatformUser user);

    /**
     * 通过用户名查询用户
     *
     * @param userName 用户名
     * @return 用户对象信息
     */
    SysPlatformUser selectUserByUserName(String userName);

    /**
     * 通过用户ID查询用户
     *
     * @param userId 用户ID
     * @return 用户对象信息
     */
    SysPlatformUser selectUserById(Long userId);

    /**
     * 根据用户ID查询用户所属角色组
     *
     * @param userName 用户名
     * @return 结果
     */
     String selectUserRoleGroup(String userName);

    /**
     * 根据用户ID查询用户所属岗位组
     *
     * @param userName 用户名
     * @return 结果
     */
     String selectUserPostGroup(String userName);

    /**
     * 校验用户名称是否唯一
     *
     * @param user 用户名称
     * @return 结果
     */
    boolean checkUserNameUnique(SysPlatformUser user);

    /**
     * 校验手机号码是否唯一
     *
     * @param user 用户信息
     * @return 结果
     */
    boolean checkPhoneUnique(SysPlatformUser user);

    /**
     * 校验email是否唯一
     *
     * @param user 用户信息
     * @return 结果
     */
    boolean checkEmailUnique(SysPlatformUser user);

    /**
     * 校验用户是否允许操作
     *
     * @param user 用户信息
     */
     void checkUserAllowed(SysPlatformUser user);

    /**
     * 校验用户是否有数据权限
     *
     * @param userId 用户id
     */
     void checkUserDataScope(Long userId);

    /**
     * 新增用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
     int insertUser(SysPlatformUser user);

    /**
     * 注册用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
     boolean registerUser(SysPlatformUser user);

    /**
     * 修改用户信息
     *
     * @param user 用户信息
     * @return 结果
     */
     int updateUser(SysPlatformUser user);

    /**
     * 用户授权角色
     *
     * @param userId 用户ID
     * @param roleIds 角色组
     */
     void insertUserAuth(Long userId, Long[] roleIds);

    /**
     * 修改用户状态
     *
     * @param user 用户信息
     * @return 结果
     */
     int updateUserStatus(SysPlatformUser user);

    /**
     * 修改用户基本信息
     *
     * @param user 用户信息
     * @return 结果
     */
     int updateUserProfile(SysPlatformUser user);

    /**
     * 修改用户头像
     *
     * @param userName 用户名
     * @param avatar 头像地址
     * @return 结果
     */
     boolean updateUserAvatar(String userName, String avatar);

    /**
     * 重置用户密码
     *
     * @param user 用户信息
     * @return 结果
     */
     int resetPwd(SysPlatformUser user);

    /**
     * 重置用户密码
     *
     * @param userName 用户名
     * @param password 密码
     * @return 结果
     */
     int resetUserPwd(String userName, String password);

    /**
     * 通过用户ID删除用户
     *
     * @param userId 用户ID
     * @return 结果
     */
     int deleteUserById(Long userId);

    /**
     * 批量删除用户信息
     *
     * @param userIds 需要删除的用户ID
     * @return 结果
     */
     int deleteUserByIds(Long[] userIds);

    /**
     * 导入用户数据
     *
     * @param userList 用户数据列表
     * @param isUpdateSupport 是否更新支持，如果已存在，则进行更新数据
     * @param operName 操作用户
     * @return 结果
     */
     String importUser(List<SysPlatformUser> userList, Boolean isUpdateSupport, String operName);

    /**
     * 批量添加用户角色
     *
     * @param user
     */
    void insertUserRole(SysPlatformUser user);


    /**
     * 新增用户角色信息
     *
     * @param userId  用户ID
     * @param roleIds 角色组
     */
    void insertUserRole(Long userId, Long[] roleIds);


    int deleteUserRoleByUserId(Long userId);

    /**
     * 获取用户列表
     * @param nickName
     * @return
     */
    List<SysPlatformUser> getUserList(String nickName);
}
