package com.vctgo.system.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.vctgo.system.domain.SysPost;

import java.util.List;

/**
 * 岗位信息 服务层
 *
 * @author vctgo
 */
 public interface ISysPostService
{
    /**
     * 查询岗位信息集合-分页
     *
     * @param post 岗位信息
     * @return 岗位列表
     */
    IPage<SysPost> selectPostPage(SysPost post);
    /**
     * 查询岗位信息集合
     *
     * @param post 岗位信息
     * @return 岗位列表
     */
     List<SysPost> selectPostList(SysPost post);

    /**
     * 查询所有岗位
     *
     * @return 岗位列表
     */
     List<SysPost> selectPostAll();

    /**
     * 通过岗位ID查询岗位信息
     *
     * @param postId 岗位ID
     * @return 角色对象信息
     */
     SysPost selectPostById(Long postId);

    /**
     * 根据用户ID获取岗位选择框列表
     *
     * @param userId 用户ID
     * @return 选中岗位ID列表
     */
     List<Long> selectPostListByUserId(Long userId);

    /**
     * 校验岗位名称
     *
     * @param post 岗位信息
     * @return 结果
     */
    boolean checkPostNameUnique(SysPost post);

    /**
     * 校验岗位编码
     *
     * @param post 岗位信息
     * @return 结果
     */
    boolean checkPostCodeUnique(SysPost post);

    /**
     * 校验岗位编码
     *
     * @param post 岗位信息
     * @return 结果
     */
    void createSysDefaultPost(SysPost post);

    /**
     * 通过岗位ID查询岗位使用数量
     *
     * @param postId 岗位ID
     * @return 结果
     */
     int countUserPostById(Long postId);

    /**
     * 删除岗位信息
     *
     * @param postId 岗位ID
     * @return 结果
     */
     int deletePostById(Long postId);

    /**
     * 批量删除岗位信息
     *
     * @param postIds 需要删除的岗位ID
     * @return 结果
     */
     int deletePostByIds(Long[] postIds);

    /**
     * 新增保存岗位信息
     *
     * @param post 岗位信息
     * @return 结果
     */
     int insertPost(SysPost post);

    /**
     * 修改保存岗位信息
     *
     * @param post 岗位信息
     * @return 结果
     */
     int updatePost(SysPost post);

    /**
     * 变更幼儿园的岗位到所属集团
     *
     * @param gardenId 集团Id
     * @param id 幼儿园Id
     * @return 结果
     */
     int multiAdjustBranchPost(Long gardenId, Long id, Long tenantType);

    /**
     * 批量解除租户与岗位的绑定关系
     *
     * @param tenantId 租户Id
     * @return 结果
     */
     int deleteUserPostByTenantId(Long tenantId);

    /**
     * 删除租户管理信息
     *
     * @param id 租户管理主键
     * @return 结果
     */
    int deleteSysDefaultPost(Long id);
}
