package com.vctgo.system.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.vctgo.system.api.domain.SysClassSetting;
import com.vctgo.system.domain.SysReClassUser;
import com.vctgo.system.domain.dto.BindClassUserDto;

import java.util.List;

/**
 * 班级配置人员信息Service接口
 *
 * @author wanglonghua
 * @date 2023-10-09
 */
public interface ISysReClassUserService
{
    /**
     * 查询班级配置人员信息
     *
     * @param id 班级配置人员信息主键
     * @return 班级配置人员信息
     */
    SysReClassUser selectSysReClassUserById(Long id);

    /**
     * 查询班级配置人员信息列表-分页
     *
     * @param sysReClassUser 班级配置人员信息
     * @return 班级配置人员信息集合
     */
    IPage<SysReClassUser> selectSysReClassUserPage(SysReClassUser sysReClassUser);

    /**
     * 查询班级配置人员信息列表
     *
     * @param sysReClassUser 班级配置人员信息
     * @return 班级配置人员信息集合
     */
    List<SysReClassUser> selectSysReClassUserList(SysReClassUser sysReClassUser);

    /**
     * 通过班级id统计绑定的用户数
     *
     * @param classId 班级id
     * @return 班级配置人员总数
     */
    Long staticsBindUserCountByClassId(Long classId, String bindUserPosition);

    /**
     * 班级配置人员明细
     *
     * @param bindClassUserDto 班级配置人员信息
     * @return 结果
     */
    BindClassUserDto getClassAllocationDetail(BindClassUserDto bindClassUserDto);

    /**
     * 新增班级配置人员信息
     *
     * @param bindClassUserDto 班级配置人员信息
     * @return 结果
     */
    int allocationSysReClassUser(BindClassUserDto bindClassUserDto);

    /**
     * 新增班级配置人员信息
     *
     * @param userId 用户ID
     * @param bindUserPosition 用户所在的岗位/用户人员类型
     * @return 结果
     */
    List<SysClassSetting> getPointUserClassList(Long userId, Integer bindUserPosition);

    /**
     * 修改班级配置人员信息
     *
     * @param sysReClassUser 班级配置人员信息
     * @return 结果
     */
    int updateSysReClassUser(SysReClassUser sysReClassUser);

    /**
     * 逻辑删除班级配置人员信息
     *
     * @param ids
     * @return 结果
     */
    int logicalDeleteSysReClassUserByIds(Long[] ids);

    /**
     * 批量删除班级配置人员信息
     *
     * @param ids 需要删除的班级配置人员信息主键集合
     * @return 结果
     */
    int deleteSysReClassUserByIds(Long[] ids);

    /**
     * 删除班级配置人员信息信息
     *
     * @param id 班级配置人员信息主键
     * @return 结果
     */
    int deleteSysReClassUserById(Long id);
}
