package com.vctgo.system.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.vctgo.common.core.web.domain.AjaxResult;
import com.vctgo.system.api.domain.SysTenant;

import java.util.List;

/**
 * 租户管理Service接口
 *
 * @author vctgo
 * @date 2022-04-11
 */
public interface ISysTenantService
{
    /**
     * 查询租户管理
     *
     * @param id 租户管理主键
     * @return 租户管理
     */
    SysTenant selectSysTenantById(Long id);

    /**
     * 查询租户管理列表-分页
     *
     * @param sysTenant 租户管理
     * @return 租户管理集合
     */
    IPage<SysTenant> selectSysTenantPage(SysTenant sysTenant);

    /**
     * 查询租户管理列表
     *
     * @param sysTenant 租户管理
     * @return 租户管理集合
     */
    List<SysTenant> selectSysTenantList(SysTenant sysTenant);

    /**
     * 查询租户管理列表
     *
     * @param sysTenant 租户管理
     * @return 租户管理集合
     */
    List<SysTenant> selectSysTenantTree(SysTenant sysTenant);

    /**
     * 新增租户管理
     *
     * @param sysTenant 租户管理
     * @return 结果
     */
    AjaxResult insertSysTenant(SysTenant sysTenant);

    /**
     * 新增租户管理
     *
     * @param sysTenant 租户管理
     * @return 结果
     */
    AjaxResult insertNewSysTenant(SysTenant sysTenant);

    /**
     * 修改租户管理
     *
     * @param sysTenant 租户管理
     * @return 结果
     */
    int updateSysTenant(SysTenant sysTenant);

    /**
     * 修改租户管理
     *
     * @param sysTenant 租户管理
     * @return 结果
     */
    int updateSysTenantRelaData(SysTenant sysTenant);

    /**
     * 新增租户管理
     *
     * @param sysTenant 租户管理
     * @return 结果
     */
    int addConfigPermission(SysTenant sysTenant);

    /**
     * 修改租户管理
     *
     * @param sysTenant 租户管理
     * @return 结果
     */
    int editConfigPermission(SysTenant sysTenant);

    /**
     * 批量删除租户管理
     *
     * @param ids 需要删除的租户管理主键集合
     * @return 结果
     */
    int deleteSysTenantByIds(Long[] ids);

    /**
     * 删除租户管理信息
     *
     * @param id 租户管理主键
     * @return 结果
     */
    int deleteSysTenantById(Long id);

    /**
     * 获取指定租户的父级租户信息
     *
     * @param tenantId 租户管理主键
     * @return 结果
     */
    SysTenant getParentTenantInfo(Long tenantId);


    /**
     * 条件查询租户管理列表
     *
     * @param sysTenant 租户管理主键
     * @return 结果
     */
    List<SysTenant> selectTenantList(SysTenant sysTenant);
}
