package com.vctgo.system.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.vctgo.common.core.text.Convert;
import com.vctgo.common.core.utils.ServletUtils;
import com.vctgo.common.mybatisplus.constant.MybatisPageConstants;
import com.vctgo.common.security.utils.SecurityUtils;
import com.vctgo.system.domain.SysReGardenBranch;
import com.vctgo.system.mapper.SysReGardenBranchMapper;
import com.vctgo.system.service.ISysReGardenBranchService;
import com.vctgo.system.util.ConstantUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * 集团幼儿园租户关联Service业务层处理
 *
 * @author wanglonghua
 * @date 2023-10-28
 */
@Slf4j
@Service
public class SysReGardenBranchServiceImpl implements ISysReGardenBranchService {
    @Autowired
    private SysReGardenBranchMapper sysReGardenBranchMapper;

    /**
     * 查询集团幼儿园租户关联
     *
     * @param id 集团幼儿园租户关联主键
     * @return 集团幼儿园租户关联
     */
    @Override
    public SysReGardenBranch selectSysReGardenBranchById(Long id) {
        return sysReGardenBranchMapper.selectById(id);
    }

    /**
     * 查询集团幼儿园租户关联列表-分页
     *
     * @param sysReGardenBranch 集团幼儿园租户关联
     * @return 集团幼儿园租户关联
     */
    @Override
    public IPage<SysReGardenBranch> selectSysReGardenBranchPage(SysReGardenBranch sysReGardenBranch) {
        Page mpPage = new Page(Convert.toLong(ServletUtils.getParameterToInt(MybatisPageConstants.PAGE_NUM), 1L)
                , Convert.toLong(ServletUtils.getParameterToInt(MybatisPageConstants.PAGE_SIZE), 10L));
        return sysReGardenBranchMapper.selectSysReGardenBranchList(mpPage, sysReGardenBranch);
    }

    /**
     * 查询集团幼儿园租户关联列表
     *
     * @param sysReGardenBranch 集团幼儿园租户关联
     * @return 集团幼儿园租户关联
     */
    @Override
    public List<SysReGardenBranch> selectSysReGardenBranchList(SysReGardenBranch sysReGardenBranch) {
        sysReGardenBranch.setId(SecurityUtils.getTenantId());
        return sysReGardenBranchMapper.selectSysReGardenBranchList(sysReGardenBranch);
    }

    /**
     * 新增集团幼儿园租户关联
     *
     * @param sysReGardenBranch 集团幼儿园租户关联
     * @return 结果
     */

    @Override
    public int insertSysReGardenBranch(SysReGardenBranch sysReGardenBranch) {
        return sysReGardenBranchMapper.insert(sysReGardenBranch);
    }

    /**
     * 修改集团幼儿园租户关联
     *
     * @param sysReGardenBranch 集团幼儿园租户关联
     * @return 结果
     */
    @Override
    public int updateSysReGardenBranch(SysReGardenBranch sysReGardenBranch) {
        return sysReGardenBranchMapper.updateById(sysReGardenBranch);
    }

    /**
     * 逻辑删除集团幼儿园租户关联
     *
     * @param ids 需要删除的集团幼儿园租户关联主键
     * @return 结果
     */
    @Override
    public int logicalDeleteSysReGardenBranchByIds(Long[] ids) {
        SysReGardenBranch sysReGardenBranch = new SysReGardenBranch();
        sysReGardenBranch.setUpdateById(SecurityUtils.getUserId());
        sysReGardenBranch.setUpdateBy(SecurityUtils.getUsername());
        sysReGardenBranch.setUpdateTime(new Date());
        sysReGardenBranch.setIsDeleted(ConstantUtils.IS_DELETED_1);
        return sysReGardenBranchMapper.logicalDeleteSysReGardenBranchByIds(ids, sysReGardenBranch);
    }

    /**
     * 批量删除集团幼儿园租户关联
     *
     * @param ids 需要删除的集团幼儿园租户关联主键
     * @return 结果
     */
    @Override
    public int deleteSysReGardenBranchByIds(Long[] ids) {
        return sysReGardenBranchMapper.deleteSysReGardenBranchByIds(ids);
    }

    /**
     * 删除集团幼儿园租户关联信息
     *
     * @param id 集团幼儿园租户关联主键
     * @return 结果
     */
    @Override
    public int deleteSysReGardenBranchById(Long id) {
        return sysReGardenBranchMapper.deleteById(id);
    }
}
