package com.vctgo.system.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.vctgo.common.core.text.Convert;
import com.vctgo.common.core.utils.ServletUtils;
import com.vctgo.common.mybatisplus.constant.MybatisPageConstants;
import com.vctgo.system.api.domain.SysReLocationArea;
import com.vctgo.system.mapper.SysReLocationAreaMapper;
import com.vctgo.system.service.ISysReLocationAreaService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
/**
 * 位置与区域关联Service业务层处理
 *
 * @author caesar
 * @date 2023-05-30
 */
@Slf4j
@Service
public class SysReLocationAreaServiceImpl extends ServiceImpl<SysReLocationAreaMapper, SysReLocationArea> implements ISysReLocationAreaService
{
    @Autowired
    private SysReLocationAreaMapper sysReLocationAreaMapper;

    /**
     * 查询位置与区域关联
     *
     * @param id 位置与区域关联主键
     * @return 位置与区域关联
     */
    @Override
    public SysReLocationArea selectSysReLocationAreaById(Long id)
    {
        return sysReLocationAreaMapper.selectById(id);
    }

    /**
     * 查询位置与区域关联列表-分页
     *
     * @param sysReLocationArea 位置与区域关联
     * @return 位置与区域关联
     */
    @Override
    public IPage<SysReLocationArea> selectSysReLocationAreaPage(SysReLocationArea sysReLocationArea)
    {
        Page mpPage =new Page(Convert.toLong(ServletUtils.getParameterToInt(MybatisPageConstants.PAGE_NUM),1L)
                ,Convert.toLong(ServletUtils.getParameterToInt(MybatisPageConstants.PAGE_SIZE),10L));
        return sysReLocationAreaMapper.selectSysReLocationAreaList(mpPage,sysReLocationArea);
    }
    /**
     * 查询位置与区域关联列表
     *
     * @param sysReLocationArea 位置与区域关联
     * @return 位置与区域关联
     */
    @Override
    public List<SysReLocationArea> selectSysReLocationAreaList(SysReLocationArea sysReLocationArea)
    {
        return sysReLocationAreaMapper.selectSysReLocationAreaList(sysReLocationArea);
    }

    @Override
    public List<SysReLocationArea> selectRelInfoListByParams(Map<String, Object> params) {
        return sysReLocationAreaMapper.selectRelInfoListByParams(params);
    }

    /**
     * 新增位置与区域关联
     *
     * @param sysReLocationArea 位置与区域关联
     * @return 结果
     */

    @Override
    public int insertSysReLocationArea(SysReLocationArea sysReLocationArea)
    {
        return sysReLocationAreaMapper.insert(sysReLocationArea);
    }

    /**
     * 修改位置与区域关联
     *
     * @param sysReLocationArea 位置与区域关联
     * @return 结果
     */
    @Override
    public int updateSysReLocationArea(SysReLocationArea sysReLocationArea)
    {
        return sysReLocationAreaMapper.updateById(sysReLocationArea);
    }

    /**
     * 批量删除位置与区域关联
     *
     * @param ids 需要删除的位置与区域关联主键
     * @return 结果
     */
    @Override
    public int deleteSysReLocationAreaByIds(Long[] ids)
    {
        return sysReLocationAreaMapper.deleteSysReLocationAreaByIds(ids);
    }

    /**
     * 删除位置与区域关联信息
     *
     * @param id 位置与区域关联主键
     * @return 结果
     */
    @Override
    public int deleteSysReLocationAreaById(Long id)
    {
        return sysReLocationAreaMapper.deleteById(id);
    }
}
